// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
/*
 * Copyright 2011 Red Hat, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

const Gio = imports.gi.Gio;
const Lang = imports.lang;

const LoginManager = imports.misc.loginManager;

const GdmUtil = imports.gdm.util;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const GnomeSession = imports.misc.gnomeSession;
const Main = imports.ui.main;

const GLib = imports.gi.GLib;

const PowerMenuButton = new Lang.Class({
    Name: 'PowerMenuButton',
    Extends: PanelMenu.SystemIndicator,

    _init: function() {
        //log("powerMenu init called");

        /* Translators: accessible name of the power menu in the login screen */
        //this.parent('system-shutdown-symbolic', _("Power"));
        this.parent(_("Power"));
        this._indicator = this._addIndicator();
        this._indicator.icon_name = 'system-shutdown-symbolic';
        this._session = new GnomeSession.SessionManager();

        this._loginManager = LoginManager.getLoginManager();
/*
        this._settings = new Gio.Settings({ schema: GdmUtil.LOGIN_SCREEN_SCHEMA });
        this._settings.connect('changed::disable-restart-buttons',
                               Lang.bind(this, this._updateVisibility));
*/
//Wyse customization. Separate power, volume and network from aggregatemenu.
//
        this.actor.connect_after('button-release-event', Lang.bind(this, this._onButtonRelease));
        this.actor.connect_after('key-release-event', Lang.bind(this, this._onKeyRelease));

/*
        this._createSubMenu();

        // ConsoleKit doesn't send notifications when shutdown/reboot
        // are disabled, so we update the menu item each time the menu opens
        this.menu.connect('open-state-changed', Lang.bind(this,
            function(menu, open) {
                if (open) {
                    this._updateHaveShutdown();
                    this._updateHaveRestart();
                    this._updateHaveSuspend();
                }
            }));
        this._updateHaveShutdown();
        this._updateHaveRestart();
        this._updateHaveSuspend();
*/
//
    },

    _updateVisibility: function() {
     /*    
        log("power button _updateVisibility called");
        let shouldBeVisible = (this._haveSuspend || this._haveShutdown || this._haveRestart);
        this.actor.visible = shouldBeVisible && !this._settings.get_boolean('disable-restart-buttons');
     */
    },

    _updateHaveShutdown: function() {
        this._loginManager.canPowerOff(Lang.bind(this, function(result) {
            this._haveShutdown = result;
            this._powerOffItem.actor.visible = this._haveShutdown;
            this._updateVisibility();
        }));
    },

    _updateHaveRestart: function() {
        this._loginManager.canReboot(Lang.bind(this, function(result) {
            this._haveRestart = result;
            this._restartItem.actor.visible = this._haveRestart;
            this._updateVisibility();
        }));
    },

    _updateHaveSuspend: function() {
        this._loginManager.canSuspend(Lang.bind(this, function(result) {
            this._haveSuspend = result;
            this._suspendItem.actor.visible = this._haveSuspend;
            this._updateVisibility();
        }));
    },

    _createSubMenu: function() {
        let item;
        item = new PopupMenu.PopupMenuItem(_("Suspend"));
        item.connect('activate', Lang.bind(this, this._onActivateSuspend));
        this.menu.addMenuItem(item);
        this._suspendItem = item;
        item = new PopupMenu.PopupMenuItem(_("Restart"));
        item.connect('activate', Lang.bind(this, this._onActivateRestart));
        this.menu.addMenuItem(item);
        this._restartItem = item;

        item = new PopupMenu.PopupMenuItem(_("Power Off"));
        item.connect('activate', Lang.bind(this, this._onActivatePowerOff));
        this.menu.addMenuItem(item);
        this._powerOffItem = item;
    },

    _onActivateSuspend: function() {
        if (!this._haveSuspend)
            return;

        this._loginManager.suspend();
    },

    _onActivateRestart: function() {
        if (!this._haveRestart)
            return;

        this._loginManager.reboot();
    },

    _onActivatePowerOff: function() {
        Main.overview.hide();
        this._session.ShutdownRemote(0);

        if (!this._haveShutdown)
            return;

        this._loginManager.powerOff();

        
    },

//Wyse customization. Separate power, volume and network from aggregatemenu.
//
    _onButtonRelease: function() {
        Main.overview.hide();
        //log("on power button release");
        GLib.spawn_command_line_sync("regset "+"System guiPowerButtonPressed true");
        this._session.ShutdownRemote(0);
    },

    _onKeyRelease: function(actor, event) {
        Main.overview.hide();
        //log("on power key release");
        GLib.spawn_command_line_sync("regset "+"System guiPowerButtonPressed true");
        this._session.ShutdownRemote(0);
    },
//

});

